# -*- coding: utf-8 -*-
#
#  This file is part of Damona software
#
#  Copyright (c) 2016 - Damona Development Team
#
#  File author(s):
#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
#
#  Distributed under the terms of the 3-clause BSD license.
#  The full license is in the LICENSE file, distributed with this software.
#
#  website: https://github.com/cokelaer/damona
#  documentation: http://damona.readthedocs.io
#
##############################################################################
import os
from spython.main import Client
from damona import bin_directory, images_directory
from damona import Registry
from damona import logger

logger.level = 10
class Pull():
    """Manage the download of images


    """
    def __init__(self, dryrun=False):
        self.registry = Registry().registry
        self.dryrun = dryrun

    def pull(self, name, output_name=None, pull_folder=images_directory, force=False):

        # the name must be valid. we use _ to separate name and version for the
        # singularity filemane but the image have a tag (version) and name
        # separated by a :

        # Our registry uses underscore. The output filename should use _ as well
        # However, for users and image stored on sylabs or singularity hub, we
        # use the character :
        #  
        registry_name = name.replace(":", "_")
        if  registry_name not in self.registry.keys():
            logger.critical("invalid image name provided: {}. Choose amongst {}".format(name,
self.registry.keys()))
            sys.exit(1)


        download_name = self.registry[registry_name]['download']
        if output_name is None:
            output_name = registry_name + ".img"

        if self.dryrun: 
            pass
        else: # pragma: no cover
            Client.pull(download_name, name=output_name, 
                pull_folder=pull_folder,
                force=force) 
            logger.info("File {} upload to {}".format(name, images_directory))

        # Now, create an alias
        path = images_directory

        _class = self.registry[registry_name]['class']
        if _class == "exe":
            # one binary name. we use the first one
            cmd = """singularity run {}/{} ${{1+"$@"}} """
            cmd = cmd.format(images_directory, output_name)
            bin_name = bin_directory /  registry_name.split("_")[0]
            if self.dryrun: #pragma: no cover
                pass
            else: #pragma: no cover
                with open(bin_name, "w") as fout:
                    fout.write(cmd)
                os.chmod(bin_name, 0o755)
                logger.info("Binary {} in {}".format(bin_name, bin_directory))
        elif _class == "env": #pragma: no cover
            pass
        elif _class == "set": # pragma: no cover
            raise ImplementedError
        else: # pragma: no cover
            raise ValueError





from damona import script
from damona import pull
import subprocess


def test_python_pull():
    p = pull.Pull(dryrun=True)
    p.pull("fastqc:0.11.9")

def test_python_registry():
    from damona import registry
    r = registry.Registry()
    r.get_list()

    from damona.recipes import __path__
    path = __path__[0] + "/fastqc"
    r.create_registry(path)



def test_help():
    cmd = "damona --help"
    subprocess.call(cmd.split())
    cmd = "damona pull --help"
    subprocess.call(cmd.split())


def test_list():
    cmd = "damona list --help"
    subprocess.call(cmd.split())

    cmd = "damona list"
    subprocess.call(cmd.split())

def test_develop():
    try:
        cmd = "damona develop"
        subprocess.call(cmd.split())
        assert False
    except:
        assert True

    from damona.recipes import __path__
    cmd = "damona develop --path {}".format(__path__[0] + "/fastq"  )
    subprocess.call(cmd.split())

def test_pull_dryrun():
    cmd = "damona pull fastqc:0.11.9 --dryrun"  
    subprocess.call(cmd.split())
    
def test_pull_wrong():
    # non existing image
    cmd = "damona pull tartuffe"  
    try:
        subprocess.call(cmd.split())    
        assert False
    except:
        assert True



# -*- coding: utf-8 -*-

###########################################################################
# Damona is a project to provide reproducible containers                  #
#                                                                         #
# Authors: see CONTRIBUTORS.rst                                           #
# Copyright © 2020  Institut Pasteur, Paris and CNRS.                     #
# See the COPYRIGHT file for details                                      #
#                                                                         #
# Damona is free software: you can redistribute it and/or modify          #
# it under the terms of the GNU General Public License as published by    #
# the Free Software Foundation, either version 3 of the License, or       #
# (at your option) any later version.                                     #
#                                                                         #
# Damona  is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY; without even the implied warranty of          #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
# GNU General Public License for more details.                            #
#                                                                         #
# You should have received a copy of the GNU General Public License       #
# along with this program (COPYING file).                                 #
# If not, see <http://www.gnu.org/licenses/>.                             #
###########################################################################
""".. rubric:: Standalone application dedicated to conversion"""
import os
import argparse
import glob
import json
import sys
import colorlog
import textwrap
import click
from damona  import version, bin_directory, images_directory
from spython.main import Client

__all__ = ["main"]

_log = colorlog.getLogger(__name__)
_log.level = "INFO"


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])



@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version=version)
def main():
    """This is the main help"""
    pass


@main.command()
@click.argument('name', type=click.STRING)
@click.option('--force', is_flag=True)
@click.option('--dryrun', is_flag=True, 
    help="If set, do not pull the image and do not create the binary alias")
@click.option('--output-directory', default=images_directory, 
    help="""Where to save the image (default: {})""".format(images_directory))
def pull(**kwargs):
    "Download an image given its name and version (optional)"
    name = kwargs['name']
    from damona.pull import Pull
    p = Pull(dryrun=kwargs['dryrun'])
    p.pull(name, pull_folder=kwargs["output_directory"],
        force=kwargs['force'])


@main.command()
@click.option('--pattern', default=None, 
    help="restrict the output list keeping those with this pattern")
def list(**kwargs):
    """List all available images"""
    from damona.registry import Registry
    modules = Registry().get_list(pattern=kwargs['pattern'])
    print(", ".join(modules))


@main.command()
@click.option('--path', required=True, 
    help="path to recipes directory where Singularity file(s) can be found")
def develop(**kwargs):
    """tools for developers only"""
    from damona.registry import Registry
    if kwargs['path']:
        modules = Registry().create_registry(kwargs['path'])






if __name__ == "__main__": #pragma: no cover
    main()
